package com.osi.notification.test;

import java.net.URI;
import java.util.concurrent.atomic.AtomicInteger;

import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.jms.Destination;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.osi.notification.NotificationHelper;
import com.osi.notification.destination.DestinationHelper;
import com.osi.notification.subscription.SubscriptionHelper;
import com.sun.tools.javac.util.Log;

@Singleton(name = "NotificationTester")
@Startup
public class SimpleNotificationTestBean implements
		SimpleNotificationTestResource {
	
	private static final Logger log = LoggerFactory
			.getLogger(SimpleNotificationTestBean.class);
	private Destination destination;
	private String queueName;
	private static final AtomicInteger subscriptionCounter = new AtomicInteger(1);

	@Override
	public void sendTestNotification(String notification) {
		log.info("INCOMING NOTIFICATION: {}", notification);
		if (destination != null) {
			try {
				NotificationHelper.sendNotification(notification, destination);
			} catch (Exception e) {
				log.error("Error sending notification", e);
			}
		} else {
			log.warn("The destination is null, cannot publish notification");
		}
	}

	@Override
	public void receive(String notification) {
		log.info("RECEIVED PUSH NOTIFICATION: {}", notification);
	}

	@Override
	public Response subscribe(UriInfo uriInfo) {
		final int subscriptionId = subscriptionCounter.getAndIncrement();
		queueName = "test.testQueue" + subscriptionId;
		final String jndiName = "queue/" + queueName;
		try {
			destination = DestinationHelper.createQueue(queueName, jndiName);
			URI endpointUrl = 
				uriInfo.getBaseUriBuilder()
					.path(SimpleNotificationTestResource.class)
					.path(SimpleNotificationTestResource.class, "receive").build();
			
			log.info("Created Queue REST Push Subscription to {}", endpointUrl.toString());
			SubscriptionHelper.createQueueRESTPushSubscription(queueName, endpointUrl.toString(), uriInfo.getBaseUri().toString());
			return Response.status(Response.Status.CREATED).build();
		} catch (Exception e) {
			log.error("ERROR!!", e);
		}
		return Response.serverError().build();
	}

	@Override
	public void unsubscribe() {
		log.info("Unsubscribe!");
		DestinationHelper.destroyQueue(queueName);
		destination = null;
	}

}
